%------------------------------------------------------------------------------
\section{BPL language}
%------------------------------------------------------------------------------

\paragraph{Usage}

\begin{verbatim}
ghci -Wall -package rh AbstractSyntax.lhs
\end{verbatim}

%if False
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Module header}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> module AbstractSyntax
> where
> import Control.Monad.Reader
>
> --import Message hiding (break)
> import Message hiding (And, break)

%endif

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: declarations}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> data Decl
>   =  Val Pat Expr
>   |  Function [(LVar, [Formal], Maybe Type, Expr)]
>   |  Types [TypeDecl]
>   |  Constr LCon Type
>   |  Exception Alt
>   |  Module LStrVar [StrFormal] (Maybe SigExpr) StrExpr    -- module/structure
>   |  Open StrExpr
>   |  OpenAbstract StrExpr                                  -- NEU
>   |  Local [Decl] [Decl]
>   |  Interface LSigVar [[LTyVar]] SigExpr
>      deriving (Show)

> data TypeDecl
>   =  Type LTyCon [Formal] Type
>   |  Datatype LTyCon [Formal] [Alt]
>   |  Data LTyCon                                           -- Datentyp ohne Konstruktoren
>      deriving (Show)

> type Alt
>   =  (LCon, Maybe Type)

> instance Pretty Decl

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: expressions}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> type LVar  =  (Loc, String)
> type LCon  =  (Loc, String)

> type Var   =  String
> type Con   =  String

> data Expr  
>   =  Bool     Loc Bool
>   |  If       Expr Expr Expr                  -- alternative
>   |  UnOp     UnOp Expr                       -- unary operator
>   |  BinOp    Expr BinOp Expr                 -- binary operator
>   |  Nat      Loc Integer                     -- numeral
>   |  String   Loc String                      -- string literal
>   |  Var      LVar                            -- variable
>   |  Let      [Decl] Expr                     -- local declaration
>   |  Fun      [Formal] Expr                   -- abstraction
>   |  RecFun   LVar [Formal] Type Expr         -- recursive abstraction
>   |  App      Expr [Actual]                   -- application
>   |  Rec      Pat Expr                        -- recursion
>   |  Con      LCon                            -- constructor
>   |  Sel      Expr Lab                        -- record selection
>   |  Record   [Row Expr]                      -- record construction
>   |  Seq      Expr Expr                       -- sequence
>   |  Ref      Expr                            -- allocation
>   |  Deref    Expr                            -- dereferencing
>   |  Assign   Expr Expr                       -- assignment
>   |  While    Expr Expr                       -- while-loop
>   |  Case     Expr [(Pat, Expr)]              -- case analysis
>   |  Throw    Expr                            -- throw an exception
>   |  Try      Expr [(Pat, Expr)]              -- catch an exception
>   |  Continue Expr Expr                       -- apply a continuation
>   |  LetCC    LVar Type Expr                  -- let current continuation

>--   |  Call     Prim Expr

>   |  QualVar  StrExpr LVar                    -- qualified variable
>   |  QualCon  StrExpr LCon                    -- qualified constructor
>   |  Class    [Slot]                          -- class expression
>   |  Object   [Slot]                          -- object expression
>   |  New      Expr                            -- object creation
>   |  Invoke   Expr Lab Expr                   -- method invocation

>   |  ArrFun   Expr LVar Expr                  -- array creation via function
>   |  ArrList  [Expr]                          -- array creation from list
>   |  Subscript Expr Expr                      -- array subscription
>   |  Typed    Expr Type                       -- explicitly typed expression
>      deriving (Show)

> instance Pretty Expr

> app e []  =  e
> app e es  =  App e es

> record [Field Nothing e] = e
> record fs                = Record fs


> data Actual
>   =  ActExpr Expr
>   |  ActType [Type]
>      deriving (Show)

> data Formal
>   =  ForPat  Pat
>   |  ForTypePat [LTyVar]
>      deriving (Show)

> data Row a
>   =  Field (Maybe Lab) a
>      deriving (Show)

> instance Pretty a => Pretty (Row a)

> data UnOp  
>   =  Neg -- Not??
>      deriving (Show)

> instance Pretty UnOp where
>   pretty Neg   = text "-"

> data BinOp 
>   =  Add | Sub | Mul | Div | Mod | Lt | Leq | Eq | Neq | Geq | Gt | And | Or | Cat
>      deriving (Show, Eq)

> instance Pretty BinOp where
>   pretty Add  =  text "+"
>   pretty Sub  =  text "-"
>   pretty Mul  =  text "*"
>   pretty Div  =  text "/"
>   pretty Mod  =  text "%"
>   pretty Lt   =  text "#<"
>   pretty Leq  =  text "=<"
>   pretty Eq   =  text "=="
>   pretty Neq  =  text "/="
>   pretty Geq  =  text ">="
>   pretty Gt   =  text ">#"
>   pretty And  =  text "&&"
>   pretty Or   =  text "||"
>   pretty Cat  =  text "^"

> data Prim
>   =  Chr | GetChar | Head | Null | Ord | PutChar | Show | Tail | Write | ReadLine | Size |
>      ReadFromFile | WriteToFile | System
>      deriving (Show)

> instance Pretty Prim where
>   pretty Chr      =  text "chr"
>   pretty GetChar  =  text "getchar"
>   pretty Head     =  text "head"
>   pretty Null     =  text "null"
>   pretty Ord      =  text "ord"
>   pretty PutChar  =  text "putchar"
>   pretty Show     =  text "show"
>   pretty Tail     =  text "tail"
>   pretty Write    =  text "write"
>   pretty ReadLine =  text "read-line"
>   pretty Size     =  text "size"
>   pretty ReadFromFile    =  text "read-from-file"
>   pretty WriteToFile     =  text "write-to-file"
>   pretty System   =  text "system"

Slots aka class declarations.

> data Slot
>   =  SlMethod LVar [Formal] (Maybe Type) Expr
>   |  SlLocal [Decl] [Slot]
>   |  SlInherit Expr
>   |  SlInheritIn LVar Expr [Slot]
>   |  SlInclude Expr
>   |  SlIncludeIn LVar Expr [Slot]
>      deriving (Show)

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: patterns}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> data Pat
>   =  PatSucceed
>   |  PatVar     LVar
>   |  PatQualCon StrExpr LCon (Maybe Pat)
>   |  PatCon     LCon (Maybe Pat)                       -- constructor application
>   |  PatTyped   Pat Type
>   |  PatRecord  [Row Pat]
>   |  PatAlt     Pat Pat
>   |  PatBoth    Pat Pat
>      deriving (Show)

> patRecord [Field Nothing p] = p
> patRecord fs                = PatRecord fs

> instance Pretty Pat

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: types}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> type LTyVar  =  (Loc, String)
> type LTyCon  =  (Loc, String)

> type TyVar   =  String
> type TyCon   =  String

> data Type
>   =  TyVar LTyVar
>   |  TyCon LTyCon
>   |  QualTyCon  StrExpr LTyCon
>   |  TyApp Type [Actual]
>   |  TyFun [Formal] Type
>   |  TyAnd Type Type
>   |  TyForall [Formal] Type
>   |  TyRecord [TyRow] (Maybe Int) -- optional row variable for type checking
>   |  TyPred LVar Type Expr
>   |  TyClass  [TySlot] (Maybe Int) -- optional slot variable for type checking
>   |  TyObject [TySlot] (Maybe Int) -- optional slot variable for type checking
>   -- for type checking
>   |  TVar Int -- unification variable
>   |  TNatString Int -- for builtin operators
>      deriving (Show)

> tyRecord [TyField Nothing Nothing t] = t
> tyRecord fs                          = TyRecord fs Nothing

Hmm. (x : t) ist nicht zulssig, da wir keine 1-Tupel
haben.

> data TyRow
>   =  TyField (Maybe Lab) (Maybe LVar) Type
>      deriving (Show)

> data TySlot
>   =  TyMethod LVar Type
>   |  TyInclude Type
>      deriving (Show)

> instance Pretty Type

> ltext (l,x) = text x

> splitTyName :: LCon -> (String, String)
> splitTyName (_,c) =
>   case break (=='#') c of
>     (d,_:n) -> (d,n)
>     _       -> (c,"")

> makeTyName :: Int -> String -> Con
> makeTyName n d = d ++ "#" ++ show n

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: kinds}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> data Kind
>   =  KType
>   |  KFun Kind Kind
>      deriving (Show)

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: modules}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> type LStrVar  =  (Loc,String)
> type StrVar   =  String

> data StrExpr
>   =  StrVar LStrVar
>   |  StrSel StrExpr LStrVar
>   |  Struct [Decl]
>   |  RecStruct LStrVar (Maybe SigExpr) [Decl]
>   |  Functor [StrFormal] StrExpr
>   |  StrApp StrExpr [StrActual]
>   |  StrTyped StrExpr SigExpr
>      deriving (Show)

> data StrActual
>   =  StrActStrExpr StrExpr
>   |  StrActType [Type]
>      deriving (Show)

> data StrFormal
>   =  StrForStrPat LStrVar SigExpr
>   |  StrForTypePat [LTyVar]
>      deriving (Show)

> instance Pretty StrExpr

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: interfaces}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> type LSigVar  =  (Loc,String)
> type SigVar   =  String

> data SigExpr
>   =  SigVar LSigVar
>   |  SigSel StrExpr LSigVar
>   |  Sig [Spec]
>   |  SigFunctor [StrFormal] SigExpr
>   |  SigFun [SigFormal] SigExpr
>   |  SigApp SigExpr [SigActual]
>   |  SigAbstract [SigFormal] SigExpr                  -- NEU
>      deriving (Show)

> instance Pretty SigExpr

Specification.

> data Spec
>   =  SpVal LVar Type
>   |  SpType LTyVar Type
>   |  SpData LCon
>   |  SpCon LCon Type
>   |  SpException Alt
>   |  SpModule LStrVar SigExpr
>   |  SpInterface LSigVar SigExpr
>   |  SpInclude SigExpr
>      deriving (Show)

> type SigFormal
>   =  [LTyVar]
>
> type SigActual
>   =  [Type]

> instance Pretty Spec

%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
\subsection{Abstract syntax: misc}
%- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

> data Loc = At { file :: String, line :: !Int, column :: !Int}
>   deriving Eq
>
> instance Show Loc where
>   show (At f l c) = f ++ ":" ++ show l ++ ":" ++ show c
> instance Pretty Loc where
>   pretty (At "" 0 0) = empty
>   pretty (At f l c)  = text f <> char ':' <> pretty l <> char ':' <> pretty c

> unknown = At "" 0 0

> data Lab   
>   =  AlNumLab  String                         -- alphanumeric label
>   |  NumLab    Integer                        -- numeric label
>      deriving (Eq, Ord, Show)

> instance Pretty Lab where
>   pretty = ppLab

> ppLab (AlNumLab s) = text s
> ppLab (NumLab   n) = pretty n
